@@ -1,6 +1,11 @@
 """Plugin declaration for ipfabric."""
+try:
+    from importlib import metadata
+except ImportError:
+    # Python version < 3.8
+    import importlib_metadata as metadata
 
-__version__ = "1.0.0"
+__version__ = metadata.version(__name__)
 
 from nautobot.extras.plugins import PluginConfig
 
