@@ -11,6 +11,9 @@ from nautobot_chatops.choices import CommandStatusChoices
 from nautobot_chatops.workers import subcommand_of, handle_subcommands
 from netutils.ip import is_ip
 from netutils.mac import is_valid_mac
+from ipfabric_diagrams import IPFDiagram, Unicast
+from ipfabric import IPFClient
+
 from .ipfabric import IpFabric, LAST
 from .context import get_context, set_context
 
@@ -21,12 +24,26 @@ IPFABRIC_LOGO_ALT = "IPFabric Logo"
 
 logger = logging.getLogger("rq.worker")
 
+# TODO: us of the IpFabric class is temporary until we complete migration to python-ipfabric library
 ipfabric_api = IpFabric(
     host_url=settings.PLUGINS_CONFIG["nautobot_chatops_ipfabric"].get("IPFABRIC_HOST"),
     token=settings.PLUGINS_CONFIG["nautobot_chatops_ipfabric"].get("IPFABRIC_API_TOKEN"),
     verify=settings.PLUGINS_CONFIG["nautobot_chatops_ipfabric"].get("IPFABRIC_VERIFY"),
 )
 
+ipfabric_client = IPFClient(
+    base_url=settings.PLUGINS_CONFIG["nautobot_chatops_ipfabric"].get("IPFABRIC_HOST"),
+    token=settings.PLUGINS_CONFIG["nautobot_chatops_ipfabric"].get("IPFABRIC_API_TOKEN"),
+    verify=settings.PLUGINS_CONFIG["nautobot_chatops_ipfabric"].get("IPFABRIC_VERIFY"),
+)
+
+ipfabric_diagram_client = IPFDiagram(
+    base_url=settings.PLUGINS_CONFIG["nautobot_chatops_ipfabric"].get("IPFABRIC_HOST"),
+    token=settings.PLUGINS_CONFIG["nautobot_chatops_ipfabric"].get("IPFABRIC_API_TOKEN"),
+    verify=settings.PLUGINS_CONFIG["nautobot_chatops_ipfabric"].get("IPFABRIC_VERIFY"),
+    timeout=settings.PLUGINS_CONFIG["nautobot_chatops_ipfabric"].get("IPFABRIC_TIMEOUT"),
+)
+
 inventory_field_mapping = {
     "site": "siteName",
     "model": "model",
@@ -464,22 +481,22 @@ def pathlookup(
             "type": "text",
             "label": "Destination IP",
         },
-        {
-            "type": "text",
-            "label": "Source Port",
-            "default": "1000",
-        },
-        {
-            "type": "text",
-            "label": "Destination Port",
-            "default": "22",
-        },
         {
             "type": "select",
             "label": "Protocol",
             "choices": protocols,
             "default": protocols[0],
         },
+        {
+            "type": "text",
+            "label": "Source Ports",
+            "default": "1000",
+        },
+        {
+            "type": "text",
+            "label": "Destination Ports",
+            "default": "22",
+        },
     ]
 
     if not all([src_ip, dst_ip, src_port, dst_port, protocol]):
@@ -513,10 +530,18 @@ def pathlookup(
         ]
     )
 
-    # only supported in IP Fabric OS version 4.0+
+    # diagrams for 4.0 - 4.2 are not supported due to attribute changes in 4.3+
     try:
-        if ipfabric_api.validate_version(ge, 4.0):
-            raw_png = ipfabric_api.get_pathlookup(src_ip, dst_ip, src_port, dst_port, protocol, snapshot_id)
+        os_version = ipfabric_client.os_version
+        if os_version and ge(os_version, "4.3"):
+            unicast = Unicast(
+                startingPoint=src_ip,
+                destinationPoint=dst_ip,
+                protocol=protocol,
+                srcPorts=src_port,
+                dstPorts=dst_port,
+            )
+            raw_png = ipfabric_diagram_client.diagram_png(unicast, snapshot_id)
             if not raw_png:
                 raise RuntimeError(
                     "An error occurred while retrieving the path lookup. Please verify the path using the link above."
@@ -525,12 +550,20 @@ def pathlookup(
                 # Note: Microsoft Teams will silently fail if we have ":" in our filename, so the timestamp has to skip them.
                 time_str = datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
                 img_path = os.path.join(tempdir, f"{sub_cmd}_{time_str}.png")
+                # MS Teams requires permission to upload files.
+                if dispatcher.needs_permission_to_send_image():
+                    dispatcher.ask_permission_to_send_image(
+                        f"{sub_cmd}_{time_str}.png",
+                        f"{BASE_CMD} {sub_cmd} {src_ip} {dst_ip} {src_port} {dst_port} {protocol}",
+                    )
+                    return False
+
                 with open(img_path, "wb") as img_file:
                     img_file.write(raw_png)
                 dispatcher.send_image(img_path)
         else:
             raise RuntimeError(
-                "Your IP Fabric OS version does not support PNG output. Please try the end-to-end-path command."
+                "PNG output for this chatbot is only supported on IP Fabric version 4.3 and above. Please try the end-to-end-path command."
             )
     except (RuntimeError, OSError) as error:
         dispatcher.send_error(error)
